# -*- makefile -*-
#*******************************************************************
#* This file is part of the Emulex Linux Device Driver for         *
#* Fibre Channel Host Bus Adapters.                                *
#* Copyright (C) 2003-2006 Emulex.  All rights reserved.           *
#* EMULEX and SLI are trademarks of Emulex.                        *
#* www.emulex.com                                                  *
#*                                                                 *
#* This program is free software; you can redistribute it and/or   *
#* modify it under the terms of version 2 of the GNU General       *
#* Public License as published by the Free Software Foundation.    *
#* This program is distributed in the hope that it will be useful. *
#* ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND          *
#* WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,  *
#* FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE      *
#* DISCLAIMED, EXCEPT TO THE EXTENT THAT SUCH DISCLAIMERS ARE HELD *
#* TO BE LEGALLY INVALID.  See the GNU General Public License for  *
#* more details, a copy of which can be found in the file COPYING  *
#* included with this package.                                     *
#*******************************************************************

VERSION = $(shell uname -r)
SRC = /lib/modules/$(VERSION)/build
LIB = /lib/modules/$(VERSION)

CFLAGS := $(shell make -C $(SRC) -s script "SCRIPT=echo \$$(CFLAGS)")
ARCH := $(shell make -C $(SRC) -s script "SCRIPT=echo \$$(ARCH)")
MODFLAGS := $(shell make CONFIG_MODULES=1 CONFIG_MODVERSIONS=1 -C $(SRC) -s script "SCRIPT=echo \$$(MODFLAGS)")
CC := $(CROSS_COMPILE_PREFIX)$(shell make -C $(SRC) -s script "SCRIPT=echo \$$(CC)")
LD := $(CROSS_COMPILE_PREFIX)$(shell make -C $(SRC) -s script "SCRIPT=echo \$$(LD)")
MODLIB := /lib/modules/${VERSION}
EXTRA_CFLAGS := $(shell make -C $(SRC) -s script "SCRIPT=echo \$$(EXTRA_CFLAGS)") 
EXTRA_CFLAGS += -I$(SRC)/include -I. -I$(LIB)/build/drivers/scsi $(MODFLAGS) 
LPFC_PACKAGE_DRIVER := $(LIB)/kernel/drivers/scsi/lpfc/lpfc.o
LPFC_INBOX_DRIVER := $(LIB)/kernel/drivers/addon/lpfc/lpfc.o
ifeq ($(KERNEL_BUILD),1)
  EXTRA_CFLAGS += -I/lib/modules/$(VERSION)/build/drivers/addon/lpfc
  LPFC_DRIVER_PATH := $(LPFC_INBOX_DRIVER)
else
  EXTRA_CFLAGS += -I/usr/src/lpfc
  LPFC_DRIVER_PATH := $(LPFC_PACKAGE_DRIVER)
endif

ifeq ($(OS),RHEL)
  EXTRA_CFLAGS += -DRHEL_FC
else
ifeq ($(OS),SLES)
  EXTRA_CFLAGS += -DSLES_FC
else
ifneq ($(OS),)
  $(error Error: either "make OS=RHEL" or "make OS=SLES" must be specified !!!)
else
ifneq ($(MAKECMDGOALS),clean)
ifneq ($(MAKECMDGOALS),install)
  $(error Error: either "make OS=RHEL" or "make OS=SLES" must be specified !!!)
endif
endif
endif
endif
  export EXTRA_CFLAGS
endif

NEWEST := $(shell perl -e '%files = map { ((stat($$_))[9] || 0) => $$_ } @ARGV; \
		($$newest, undef) = (sort { $$b <=> $$a } (keys %files)); \
		print "$$files{$$newest}";' /usr/src/lpfc/*.h \
		$(SRC)/drivers/addon/lpfc/*.h)

build all: .depend .lpfc lpfcmpl.o

%.o : %.c
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) -c $<

# We define the loader script first in the dependency list to match that
# up with the "-T" option in the expansion of $^
lpfcmpl.o: mpl-2.4.ld mpl_shim.o mpl_udev.o mpl_init.o mpl_proc.o mpl_default.o    \
	mpl_ds400.o mpl_msa1000.o mpl_mem.o mpl_eva.o mpl_xp.o mpl_hpaa.o mpl_jbod.o mpl_util.o
	$(LD) -r -o $@ $^

install: lpfcmpl.o
	perl lpfc-wrap-2.4.pl --install $(LPFC_DRIVER_PATH)
	install -d $(LIB)/kernel/drivers/scsi/lpfc/
	cp lpfcmpl.o  $(LIB)/kernel/drivers/scsi/lpfc/lpfcmpl.o
	depmod -a

clean:
	rm -f *.o .depend .lpfc *.map *.mod.c
	if [ -e $(LPFC_PACKAGE_DRIVER) ] ; then perl lpfc-wrap-2.4.pl --uninstall $(LPFC_PACKAGE_DRIVER) ; fi
	if [ -e $(LPFC_INBOX_DRIVER) ] ; then perl lpfc-wrap-2.4.pl --uninstall $(LPFC_INBOX_DRIVER) ; fi
	depmod -a

lpfc_init.o: /usr/src/lpfc/lpfc.conf

# when the base driver is upgraded, we need to rebuild to ensure we're compiled
# against the lpfc driver's latest header files.  The dependencies generated
# in .depend include the base driver's header files.  But the time stamps on
# these header files are inherited from the package time, not the install time.
# So we use ".lpfc" to detect changes is the base driver header file
# timestamps.
.lpfc: $(NEWEST)
	@echo Newest header is $(NEWEST)
	touch -r $(NEWEST) .lpfc
	touch mpl.h

# we need to mplize lpfc.conf before compiling mpl_init.c.  so we sneak it
# into dependency processing instead of during install.
.depend: Makefile
	perl mplize_conf.pl /etc/lpfc.conf
	@echo Building dependencies
	@for i in $(wildcard *.[cS]); do $(CC) -MM -MF $$i.d $(CFLAGS) $(EXTRA_CFLAGS) $$i || exit 1 ; done
	@cat *.d > .depend
	@rm -f *.d

ifneq ($(wildcard .depend),)
include .depend
endif
