#*******************************************************************
#* This file is part of the Emulex Linux Device Driver for         *
#* Fibre Channel Host Bus Adapters.                                *
#* Copyright (C) 2003-2006 Emulex.  All rights reserved.           *
#* EMULEX and SLI are trademarks of Emulex.                        *
#* www.emulex.com                                                  *
#*                                                                 *
#* This program is free software; you can redistribute it and/or   *
#* modify it under the terms of version 2 of the GNU General       *
#* Public License as published by the Free Software Foundation.    *
#* This program is distributed in the hope that it will be useful. *
#* ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND          *
#* WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,  *
#* FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE      *
#* DISCLAIMED, EXCEPT TO THE EXTENT THAT SUCH DISCLAIMERS ARE HELD *
#* TO BE LEGALLY INVALID.  See the GNU General Public License for  *
#* more details, a copy of which can be found in the file COPYING  *
#* included with this package.                                     *
#*******************************************************************
# This Makefile selects either the 2.4 or 2.6 Makefile
######################################################################

#$Id: Makefile-2.6.mk 1005 2006-01-27 17:27:22Z hdaschbach $

ifeq (,$(KERNELVERSION))
   KERNELVERSION := $(shell uname -r)
endif

ifeq (,$(MODLIB))
   MODLIB := /lib/modules/${KERNELVERSION}
endif

ifeq (,$(BASEDIR))
   BASEDIR := $(MODLIB)/build
endif

# Follow /lib/module/`uname -r`/build to find kernel source directory
# Unroll symlinks.  And resolve relative paths (SLES) or just accept
# absolute paths (Red Hat)
ifeq (,$(KERNELSRC))
   KERNELSRC := $(shell perl -ne 'sub BEGIN {  \
	  $$base = "$(BASEDIR)";  \
	  while (-l $$base) {  \
	    $$base = readlink $$base;  \
	  };  \
	}  \
	if (/(^KERNELSRC.*:=\s*)(\S+)/) {  \
	  $$name = $$2;  \
	  if ($$name !~ "^/") {  \
	    $$name = "$$base/$$name";  \
	  while ($$name =~ "[^/]+/\.\./") {  \
	    $$name =~ s@[^/]+/\.\./@@; };  \
	  }  \
	  print "$$name", "\n";  \
	  exit 0;  \
	}' $(BASEDIR)/Makefile)
endif

# Prefer lpfc module kit to kernel tree.  But build from kernel lpfc
# driver if there's no lpfc kit installed.
ifeq (,$(LPFCSRC))
   LPFCSRC := $(shell perl -e ' \
	foreach(qw(/usr/src/lpfc $(KERNELSRC)/drivers/scsi/lpfc  \
		   $(KERNELSRC)/drivers/scsi)) {  \
	  if (-f "$$_/lpfc_hw.h") { print; exit 0; }  \
	}')
endif

ifneq ($(GCOV),)
  EXTRA_CFLAGS += -fprofile-arcs -ftest-coverage
  EXTRA_CFLAGS += -O0
  export EXTRA_CFLAGS
endif

ifeq ($(BUILD_NO_DEBUG),)
  EXTRA_CFLAGS += -DDFC_DEBUG
  export EXTRA_CFLAGS
endif

EXTRA_INCLUDE :=  -Iinclude -Iinclude2 -Iinclude/asm/mach-generic \
		  -Iinclude2/asm/mach-generic \
		  -I$(KERNELSRC)/include
ifeq ($(OS),)
	OS := $(shell rpm -q sles-release >/dev/null 2>&1 && echo SLES)
endif
ifeq ($(OS),)
	OS := $(shell rpm -q redhat-release >/dev/null 2>&1 && echo RHEL)
endif
ifeq ($(OS),RHEL)
  EXTRA_CFLAGS += -DRHEL_FC
  MODPOST := $(MODLIB)/build/scripts/mod/modpost
  MODPOST_FLAGS := -m
  ifeq ($(KERNEL_BUILD),1)
    $(error Error: KERNEL_BUILD=1 not supported for OS=RHEL.)
  endif
else
ifeq ($(OS),SLES)
  EXTRA_CFLAGS += -DSLES_FC
  MODPOST := $(MODLIB)/build/scripts/modpost
  EXTRA_INCLUDE := $(EXTRA_INCLUDE) -I$(MODLIB)/source/include
  export MODVERDIR := $(if $(KBUILD_EXTMOD),$(firstword $(KBUILD_EXTMOD))/).tmp_versions
else
ifneq ($(OS),)
  $(error Error: either "make OS=RHEL" or "make OS=SLES" must be specified !!!)
else
ifneq ($(MAKECMDGOALS),clean)
ifneq ($(MAKECMDGOALS),install)
  $(error Error: either "make OS=RHEL" or "make OS=SLES" must be specified !!!)
endif
endif
endif
endif

  export EXTRA_CFLAGS
endif
export OS

CFLAGS += -I$(LPFCSRC)
LPFC_DRIVER_PATH := $(MODLIB)/kernel/drivers/scsi/lpfc
MPL_MODULE_PATH  := $(MODLIB)/kernel/drivers/scsi/lpfcmpl

# KERNELRELEASE is in the second pass of the 2.6 make process
ifneq (,$(KERNELRELEASE))

obj-m := lpfcmpl.o

lpfcmpl-objs := mpl-2.6.ld mpl_shim.o mpl_udev.o mpl_init.o mpl_default.o mpl_proc.o \
            mpl_ds400.o mpl_msa1000.o mpl_mem.o mpl_eva.o mpl_xp.o mpl_hpaa.o mpl_jbod.o mpl_util.o

lpfc.o: $(LPFC_DRIVER_PATH)/lpfc.ko lpfcmpl.ko
	cp $(LPFC_DRIVER_PATH)/lpfc.ko .
	perl lpfc-wrap-2.6.pl --install ./lpfc.o

else

# Building for 2.6 kernel

clean-files := .depend .lpfc *.o *.ko *.mod.c .*.cmd .*.flags .tmp_versions

.PHONY: all build install

all build: .lpfc
	$(MAKE) -C $(BASEDIR) SUBDIRS=`pwd` modules

install: build lpfc.ko
	@rm -f /lib/modules/$(KERNELVERSION)/kernel/drivers/scsi/lpfcmpl.ko
	install -d $(MPL_MODULE_PATH)
	install -c lpfcmpl.ko $(MPL_MODULE_PATH)
	install -c lpfc.ko $(LPFC_DRIVER_PATH)
	depmod -a

lpfc-wrapped.o: $(LPFC_DRIVER_PATH)/lpfc.ko lpfcmpl.ko
	cp $(LPFC_DRIVER_PATH)/lpfc.ko ./lpfc-wrapped.o
	perl lpfc-wrap-2.6.pl --install ./lpfc-wrapped.o

lpfc-wrapped.mod.o: lpfc-wrapped.mod.c
	(dir=`pwd`; cd $(BASEDIR); \
	            $(CC)  -nostdinc -iwithprefix include -D__KERNEL__ -Wall \
		            -DMODULE $(EXTRA_INCLUDE) \
			   -c -o $$dir/$@ $$dir/$<)

.tmp_versions/lpfc-wrapped.mod:
	echo lpfc-wrapped.ko > $@
	echo mpl_shim.o mpl_init.o >> $@

lpfc-wrapped.mod.c: lpfc-wrapped.o lpfcmpl.o .tmp_versions/lpfc-wrapped.mod
	$(MODPOST) $(MODPOST_FLAGS) -i $(MODLIB)/build/Module.symvers \
		      ./lpfcmpl.o \
	              ./lpfc-wrapped.o

lpfc.ko: lpfc-wrapped.o lpfc-wrapped.mod.o
	$(LD) -r -o $@ $^

clean:
	rm -rf $(clean-files) 
	if [ -e $(LPFC_DRIVER_PATH)/lpfc.ko ] ; then perl lpfc-wrap-2.6.pl --uninstall $(LPFC_DRIVER_PATH)/lpfc.ko ; fi
	depmod -a

# silence warning about missing .mpl-2.6.ld.cmd file.  The loader
# script is part of the module dependency tree.  But it is not built
# from another source.
.mpl-2.6.ld.cmd:
	echo > .mpl-2.6.ld.cmd

# when the base driver is upgraded, we need to rebuild to ensure we're compiled
# against the lpfc driver's latest header files.  But the time stamps on
# these header files are inherited from the package time, not the install time.
# So we use ".lpfc" to detect changes is the base driver header file
# timestamps.
.lpfc: $(LPFCSRC)/lpfc_hw.h .mpl-2.6.ld.cmd
	touch -r $< -d "+1 second" .lpfc
	touch mpl_util.c
endif
