#!/usr/bin/perl -w
#*******************************************************************
#* This file is part of the Emulex Linux Device Driver for         *
#* Fibre Channel Host Bus Adapters.                                *
#* Copyright (C) 2003-2006 Emulex.  All rights reserved.           *
#* EMULEX and SLI are trademarks of Emulex.                        *
#* www.emulex.com                                                  *
#*                                                                 *
#* This program is free software; you can redistribute it and/or   *
#* modify it under the terms of version 2 of the GNU General       *
#* Public License as published by the Free Software Foundation.    *
#* This program is distributed in the hope that it will be useful. *
#* ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND          *
#* WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,  *
#* FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE      *
#* DISCLAIMED, EXCEPT TO THE EXTENT THAT SUCH DISCLAIMERS ARE HELD *
#* TO BE LEGALLY INVALID.  See the GNU General Public License for  *
#* more details, a copy of which can be found in the file COPYING  *
#* included with this package.                                     *
#*******************************************************************

use Getopt::Long;

$program_name = $0;

sub usage {
  my $msg = shift(@_);
  defined $msg && print STDERR $msg, "\n";
  print STDERR "usage: $program_name { --install | --uninstall } \\\n";
  print STDERR "                  <input filename> [ <output filename> ]\n";
  exit 1;
}

my $opt_i = '';
my $opt_u = '';
my $opt_v = '';

GetOptions("install" => \$opt_i,
	   "i" => \$opt_i,
	   "uninstall" => \$opt_u,
	   "u" => \$opt_u,
	   "verbose" => \$opt_v,
	   "v" => \$opt_v);

$iname = shift(@ARGV);
$oname = shift(@ARGV);

usage("Need to specify --install or --uninstall") unless $opt_i || $opt_u;
usage("Missing filename") unless $iname;
usage("Cannot access $iname") unless -w $iname;
$oname || ($oname = $iname);

@symbols = qw(scsi_host_alloc
	      scsi_remove_host
              scsi_host_put
	      scsi_add_host
	      scsi_block_requests
	      scsi_unblock_requests
              scsi_track_queue_full
              scsi_adjust_queue_depth
              pci_register_driver
              pci_unregister_driver
              fc_target_block
              fc_target_unblock);
$installed_re = $symbols[0] . "_wrapped";


# Convert symbols from the form scsi_host_alloc_Rsmp_4853a9b7 to
# scsi_host_alloc_wrapped

sub install {
  $opts = "";
  foreach $entry (@symbols) {
    $opts = $opts . " --redefine-sym $entry=$entry" . "_wrapped";
  }

  $cmd = "objcopy $opts $iname $oname";
  print "$cmd\n" if $opt_v;
  system("$cmd");
}

# Convert symbols from the form scsi_register_wrapped to
# scsi_register_Rsmp_4853a9b7

use File::Temp ("tempfile");

sub uninstall {
  $found_wrapped = 0;
  # First check to see if we even need to uninstall.
  open SYMBOLS, "objdump -t $iname |" or die "Cannot run objdump $!.\n";
  while (<SYMBOLS>) {
    if (m/$installed_re/) {
      $found_wrapped = 1;
      last;
    }
  }
  close SYMBOLS;

  if (!$found_wrapped) {
    print "Not installed.  No need for further processing.\n" unless !$opt_v;
    exit 0;
  }

  $opts = "";
  foreach $entry (@symbols) {
    $opts = $opts . " --redefine-sym $entry" . "_wrapped=$entry";
  }

  $cmd = "objcopy $opts $iname $oname";
  print "$cmd\n" if $opt_v;
  system("$cmd");
}

install if $opt_i;
uninstall if $opt_u;
